/* CairoApplet.java
 * ================
 * Mike Jahr, 8/16/99
 * Machine Translation group, WS99
 * Center for Language and Speech Processing
 */

import javax.swing.JFrame;
import java.awt.Dimension;
import javax.swing.JMenu;
import java.io.File;

/** 
 * Wrapper for Cairo which lets it be used as a standalone application.
 * @author Mike Jahr
 * @see Cairo
 * @see CairoWrap
 * @see CairoApplet
*/
public class CairoApp extends JFrame implements CairoWrap {
    private Cairo c = null;
    
    /**
     * Main method for using Cairo in application mode. Creates a new
     * <code>CairoApp</code> object, and then attempts to open
     * any files given as command-line arguments.
     */
    public static void main(String args[]) {
	CairoApp ca = new CairoApp();
	if (args.length > 0) {
	    ca.getCairo().openFile(new File(args[0]));
	}
    }
    
    /** 
     * Creates and initializes a {@link Cairo Cairo} object, then calls
     * {@link Cairo#init() Cairo.init}.
     * @see Cairo#init()
     */
    public CairoApp() {
	super("Cairo");
	setSize(Cairo.DEFAULT_WIDTH, Cairo.DEFAULT_HEIGHT);
	setVisible(true);
	c = new Cairo(this);
	c.init();
	validate();
    }
    /** 
     * Wrapper for {@link Cairo#getMinimumSize()
     * Cairo.getMinimumSize}.
    */
    public Dimension getMinimumSize() {
	return c.getMinimumSize();
    }
    /** 
     * Wrapper for {@link Cairo#getPreferredSize() 
     * Cairo.getPreferredmSize}.
     */
    public Dimension getPreferredSize() {
	return c.getPreferredSize();
    }
    
    /**
     * Required by the {@link CairoWrap CairoWrap} interface; returns a
     * {@link FileMenuApp FileMenuApp}.
     */
    public JMenu createFileMenu(Cairo inc) {
	return new FileMenuApp(inc);
    }
    /**
     * Required by the {@link CairoWrap CairoWrap} interface; returns the
     * associated {@link Cairo Cairo} object.
     */
    public Cairo getCairo() {
	return c;
    }
}
