/* CairoApplet.java
 * ================
 * Mike Jahr, 8/16/99
 * Machine Translation group, WS99
 * Center for Language and Speech Processing
 */

import javax.swing.JApplet;
import javax.swing.JMenu;
import java.net.URL;
import java.io.*;
import java.util.StringTokenizer;

/** 
 * Wrapper for Cairo which lets it be used as an applet.
 * This class can be embedded in web pages to make alignments
 * publicly viewable. Note, however, that Cairo makes extensive use of the Swing
 * libraries introduced in Java 2 (1.2), and as of this writing most
 * browsers (eg Netscape 4.x) do not support Swing classes.
 * To use Cairo from within a browser, see
 * <a href=http://java.sun.com/products/plugin>
 * http://java.sun.com/products/plugin</a>, where Sun provides a 
 * Java 2 plug-in for Netscape on many platforms.
 * @author Mike Jahr
 * @see Cairo
 * @see CairoWrap
*/
public class CairoApplet extends JApplet implements CairoWrap {
    private Cairo c = null;
    
    /**
     * Required by the {@link CairoWrap CairoWrap} interface; returns a
     * {@link FileMenuApplet FileMenuApplet}.
     */
    public JMenu createFileMenu(Cairo inc) {
	return new FileMenuApplet(this, Cairo.DEFAULT_ALIGN_DIR);
    }
    /**
     * Required by the {@link CairoWrap CairoWrap} interface; returns the
     * associated {@link Cairo Cairo} object.
     */
    public Cairo getCairo() {
	return c;
    }
    
    //////////////////// APPLET METHODS ////////////////////
    
    public CairoApplet() {
	super();
    }
    
    /** 
     * Creates and initializes a {@link Cairo Cairo} object, then calls
     * {@link Cairo#init() Cairo.init()}.
     * @see Cairo#init()
     */
    public void init() {
	super.init();
	c = new Cairo(this);
	c.init();
	validate();
    }

    /** Attempts to open any default alignment file we received as a parameter. */
    public void start() {
	super.start();
	
	String fname = null;
	try {
	    fname = getParameter("align");
 	} catch(Exception ignored) { };
	
	if (fname != null) {
	    c.openURL(getDocumentBase(), fname);
	}
    }
    
    /**
     * Wrapper for {@link Cairo#stop() Cairo.stop()}.
     */
    public void stop() {
	c.stop();
	super.stop();
    }
    
    /** 
     * Returns version and author information according to Cairo. 
     * @see Cairo#BYLINE
     * @see Cairo#VERSION
     */
    public String getAppletInfo() {
	return "Title: " + Cairo.VERSION + "\nAuthor: " + Cairo.BYLINE + 
	    "\nAn applet to display and analyze sentence alignments.";
    }
    
    /**
     * Returns a brief descripton of recognized parameters.
     */
    public String[][] getParameterInfo() {
	String[][] info = {
	    { "align", "path string", "The .aln file to be displayed " + 
	      "on startup." },
	    { "dir", "alignment directory", "Colon-delimited list of " + 
	      "directories to be searched for alignment files." }
	};
	return info;
    }
    
    /**
     * Wrapper for {@link Cairo#getAppletInfo() Cairo.getAppletInfo()}.
     *
    public String getAppletInfo() {
	return c.getAppletInfo();
    }
    **
     * Wrapper for {@link Cairo#getParameterInfo() Cairo.getParameterInfo()}.
     *
    public String[][] getParameterInfo() {
	return c.getParameterInfo();
    }
    */
}
