/* FileMenuApp.java
 * ================
 * Mike Jahr, 8/16/99
 * Machine Translation group, WS99
 * Center for Language and Speech Processing
 * 
 * File menu class for Cairo app.
 */

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import java.awt.event.*;
import java.net.URL;
import java.io.*;
import java.util.StringTokenizer;

/** File menu class for Cairo app.
 *  Consists of three choices: open, open url, and exit.
 *  Because it is only used when Cairo is run as an application,
 *  it takes advantage of a JFileChooser to let users select
 *  alignment files to open.
 *  
 *  @author Mike Jahr
 *  @see FileMenuApplet
 *  @see Cairo
 */
public class FileMenuApp extends JMenu implements ActionListener {
    private Cairo master;
    private JFileChooser fc;
    
    /** Creates a FileMenu which operates on the given Cairo. */
    public FileMenuApp(Cairo c) {
	super("File");
	master = c;
	
	JMenuItem item;
	//file.setMnemonic(KeyEvent.VK_F);
	
	item = new JMenuItem(OPEN); //, KeyEvent.VK_O);
	item.addActionListener(this);
	item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, 
						   ActionEvent.ALT_MASK));
	add(item);
	
	item = new JMenuItem(OPEN_URL); //, KeyEvent.VK_O);
	item.addActionListener(this);
	add(item);

	addSeparator();
	
	item = new JMenuItem(EXIT); //, KeyEvent.VK_O);
	item.addActionListener(this);
	add(item);
	
	fc = new JFileChooser(Cairo.DEFAULT_ALIGN_DIR);
	fc.setFileFilter(new AlignmentFilter());
	fc.addChoosableFileFilter(fc.getAcceptAllFileFilter());
	
	/*
	  item = new JMenuItem(FILE_CLOSE); //, KeyEvent.VK_W);
	  item.addActionListener(this);
	  item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, 
	  ActionEvent.ALT_MASK));
	  add(item);
	*/
    }
    
    /** Method required by ActionListener interface. */
    public void actionPerformed(ActionEvent ae) {
	String result;
	JMenuItem source = (JMenuItem)(ae.getSource());
	
	if (source.getText().equals(OPEN)) {
	    if (fc.showOpenDialog(master) == fc.APPROVE_OPTION) {
		master.openFile(fc.getSelectedFile());
	    }
	} else if (source.getText().equals(OPEN_URL)) {
	    result = JOptionPane.
		showInputDialog(master, "Please specify the URL of an alignment " 
				+ "(.aln) file to open:", "Open", 
				JOptionPane.QUESTION_MESSAGE);
	    if (result != null && result.length() > 0) {
		//if (!result.startsWith(ALIGN_DIR))
		//	result = ALIGN_DIR + result;
		//if (!result.endsWith(ALIGN_SUFFIX))
		//	result = result + ALIGN_SUFFIX;
		master.openURL(result);
	    }
	} else if (source.getText().equals(EXIT)) {
	    System.exit(0);
	}
    }
    
    private static final String OPEN_URL = "Open url...";
    private static final String CLOSE = "Close";
    private static final String OPEN = "Open...";
    private static final String EXIT = "Exit";
    
    /**
     * File filter for alignment (.aln) files.
     * @see Cairo#ALIGN_SUFFIX
     */
    protected static final class AlignmentFilter extends FileFilter {
	public boolean accept(File f) {
	    return (f.isDirectory() || 
		    f.getName().endsWith(Cairo.ALIGN_SUFFIX));
	}
	
	public String getDescription() {
	    return ("Alignment files (*" + Cairo.ALIGN_SUFFIX + ")");
	}
    }
}
