/**
 * Link class
 * Noah A. Smith, 13 July 1999
 * Last edited 15 July 1999
 * Statistical Machine Translation Team at WS '99
 **/

import java.awt.Color;

/** A connection between two Words; displays visually as a colored line.
    @author Noah A. Smith, Mike Jahr
    @see Web
    @see Word
*/
public class Link {
    protected Word sword;
    protected Word tword;
    public Color activeColor; /** The color of the link when it is selected. */
    public Color passiveColor;  /** The color of the link when it is not selected. */
    private boolean[] types = new boolean[Web.MAX_ALIGNMENTS];

    /** Create a link between source word 's' and target word 't'. */
    Link(Word s, Word t){
	sword = s;
	tword = t;
	sword.addLink(this);
	tword.addLink(this);
    }

    /** Return the source word. */
    public Word source(){
	return sword;
    }

    /** Return the target word. */
    public Word target(){
	return tword;
    }

    /** Get the types of the link (this refers to which alignments it is relevant to). */
    public boolean[] getTypes(){
	return types;
    }

    /** Set the type of the link. */
    public void addType(int t){
	if(t < Web.MAX_ALIGNMENTS)
	    types[t] = true;
    }

    /** Update the active and passive colors based on the Web's knowledge of the number of alignments. */
    public void updateColors() {
	activeColor = Web.getActiveColor(types);
	passiveColor = Web.getPassiveColor(types);
    }
}
