/*
 * NullWord class
 * Noah A. Smith, 13 July 1999
 * Last edited 21 July 1999, NAS
 * Statistical Machine Translation Team at WS '99
 */

import java.io.StreamTokenizer;
import java.io.IOException;

/** The empty, undisplayed word to which other words may align.
    @author Noah A. Smith
    @see Word
*/
public class NullWord extends Word {
    private Probability p0 = new Probability();

    /** Instantiate the null word. */
    public NullWord(){
	super("<NULL>", -1, null);
    }

    /** True. */
    public boolean isNull() {
	return true;
    }
    
    /** Set the value of the probability of not inserting a null word (p0) to 'p'. */
    public void setP0(double p){
	p0 = new Probability(p, true);
    }

    /** Return the value of the probability of not inserting a null word (p0). */
    public double P0(){
	return p0.val();
    }

    /** Return the value of the probability of inserting a null word (p1). */
    public double P1(){
	return (double)(1) - p0.val();
    }
    
    /** Read the value of p0 from the StreamTokenizer 'st'. */
    public void readProb(StreamTokenizer st) throws FormatException {
	//p0.read(st, true);
	int tok;
	boolean done = false;
	try {	    
	    while (!done) {
		tok = st.nextToken();
		if (tok == st.TT_NUMBER)
		    throw new FormatException("Zut alors!  Nu  01");
		if (tok == st.TT_EOF)
		    throw new FormatException("Unexpected end of file.");
		if(tok == st.TT_WORD){
		    p0 = new Probability(Double.valueOf(st.sval).doubleValue(), true);
		    done = true;
		}
	    }
	} catch (IOException ioe) {
	    throw new FormatException(ioe.getMessage());
	}
    }

    /** Clear the NULL translations. */
    public void clear(){
	translations.clear();
    }
}

