/* OptionMenu.java
 * ===============
 * Mike Jahr, 8/13/99
 * Machine Translation group, WS99
 * Center for Language and Speech Processing
 * 
 * Option menu class for Cairo.
 */

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.net.URL;
import java.io.*;
import java.util.StringTokenizer;

/**
 * Implements the Option menu for Cairo.
 * Lets users customize various aspects of the Cairo interface, including
 * which and how many tables to show, vertical or horizontal alignment,
 * displaying reference translations, and font selection.
 * @author Mike Jahr
 * @see Cairo
 */
public class OptionMenu extends JMenu 
    implements ActionListener, ItemListener {
    private Cairo master;
    private JMenu streams;
    
    private String[] fontList;
    //private JMenuItem showHuman;
    //private JCheckBoxMenuItem showAll;
    
    /** Creates an OptionMenu for the given Cairo. */
    public OptionMenu(Cairo inc) {
	super("Options");
	master = inc;
	
	JMenuItem item;
	ButtonGroup bg;
	//setMnemonic(KeyEvent.VK_O);
	fontList = null;
	
	JMenu fm = new JMenu(FONT_SUB);
	
	item = new JMenuItem(SOURCE_FONT);
	item.addActionListener(this);
	fm.add(item);
	
	item = new JMenuItem(TARGET_FONT);
	item.addActionListener(this);
	fm.add(item);
	
	item = new JMenuItem(TABLE_FONT);
	item.addActionListener(this);
	fm.add(item);
	
	addSeparator();
	
	item = new JMenuItem(SHOW_REFERENCE);
	item.addActionListener(this);
	add(item);
	
	item = new JCheckBoxMenuItem(SHOW_ALL);
	item.setSelected(false);
	item.addItemListener(this);
	add(item);
	
	addSeparator();
	
	bg = new ButtonGroup();
	item = new JRadioButtonMenuItem(SHOW_HORIZ);
	bg.add(item);
	item.setSelected(true);
	item.addActionListener(this);
	add(item);
	
	item = new JRadioButtonMenuItem(SHOW_VERT);
	bg.add(item);
	item.addActionListener(this);
	add(item);
	
	addSeparator();
	
	bg = new ButtonGroup();
	item = new JRadioButtonMenuItem(ONE_UP);
	bg.add(item);
	item.addActionListener(this);
	add(item);
	
	item = new JRadioButtonMenuItem(TWO_UP);
	bg.add(item);
	item.addActionListener(this);
	add(item);
	
	item = new JRadioButtonMenuItem(FOUR_UP);
	bg.add(item);
	item.addActionListener(this);
	item.setSelected(true);
	add(item);
	
	addSeparator();
	
	streams = new JMenu("Streams");
	streams.setEnabled(false);
	add(streams);
	/*
	  Vector v = new Vector();
	  int i = 0;
	  while (i++ < 4)
	  v.add("");
	  updateStreams(v);
	*/
	//streams.setEnabled(true);
	
	/*
	  item = new JMenuItem(COLOR_SELECTION); //, KeyEvent.VK_O);
	  item.addActionListener(this);
	  colors.add(item);
	  item = new JMenuItem(COLOR_LINKS); //, KeyEvent.VK_W);
	  item.addActionListener(this);
	  colors.add(item);
	  item = new JMenuItem(COLOR_DEFAULT);
	  item.addActionListener(this);
	  colors.add(item);
	*/
    }

    private Font chooseFont(String m) {
	if (fontList == null)
	    buildFontList();
	return null;
    }

    private void buildFontList() {
	fontList = GraphicsEnvironment.getLocalGraphicsEnvironment().
	    getAvailableFontFamilyNames();
	
	JMenu fm = new JMenu("Fonts");
	JMenu m = null;
	JMenuItem mi;
	/*
	for (int i = 0; i < fontList.length; i++) {
	    if (i % 20 == 0) {
		if (m != null)
		    fm.add(m);
		m = new JMenu("" + (i / 20));
		//System.err.println("menu: " + (i / 20));
	    }
	    mi = new JMenuItem(fonts[i].getFontName());
	    mi.addActionListener(al);
	    m.add(mi);
	    
	    System.err.println("Added " + i + "=" + fonts[i]);
	}
	fm.add(m);
	*/
    }
    /** 
     * In case the Swing people ever get around to implementing this
     * feature, the Option menu will be a tear-off menu.
    */
    public boolean isTearOff() {
	return true;
    }
    /**
     * Required by the <code>ItemEventListener</code> interface; responds to
     * <code>ItemEvents</code>.
     */
    public void itemStateChanged(ItemEvent ie) {
	JMenuItem source = (JMenuItem)(ie.getSource());
	if (source.getText().equals(SHOW_ALL)) {
	    /*int n = (ie.getStateChange() == ie.SELECTED) ?
	      Sentence.REFERENCE : Sentence.TARGET;*/
	    master.getTrans().web.setShowAlt(ie.getStateChange() == ie.SELECTED);
	} else {
	    StreamMenuItem smi = null;
	    try { smi = (StreamMenuItem) source; } catch (Exception ignored) {}
	    if (smi != null) {
		master.setStreamVisible(smi.stream, 
				   ie.getStateChange() == ie.SELECTED);
		//System.err.println("streamVisible: " + smi.stream + ", " +
		//	       (ie.getStateChange() == ie.SELECTED));
	    }
	}
	
    }
    
    /**
     * Required by the <code>ActionListener</code> interface; responds to
     * <code>ActionEvents</code>.
     */
    public void actionPerformed(ActionEvent ae) {
	JMenuItem source = (JMenuItem)(ae.getSource());
	String action = source.getText();
	
	if (action.equals(ONE_UP)) {
	    master.showOneUp();
	} else if (action.equals(TWO_UP)) {
	    master.showTwoUp();
	} else if (action.equals(FOUR_UP)) {
	    master.showFourUp();
	} else if (action.equals(SHOW_REFERENCE)) {
	    master.getTrans().web.swapTarget();
	    source.setText(SHOW_MACHINE);
	} else if (action.equals(SHOW_MACHINE)) {
	    master.getTrans().web.swapTarget();
	    source.setText(SHOW_REFERENCE);
	} else if (action.equals(SHOW_HORIZ)) {
	    master.showHoriz();
	} else if (action.equals(SHOW_VERT)) {
	    master.showVert();
	    //} else if (action.equals(SET_FONT)) {
	    //FontChooser.ask(;
	}
	/*
	    String s = "Action event detected.\n"
	    + "    Event source: " + source.getText();
	    //		+ " (an instance of " + getClassName(source) + ")";
	    System.err.println(s);*/
    }
    
    /**
     * Updates the Streams menu to reflect the streams present in the
     * given string. If it is null, the menu is disabled.
     * @see Cairo#updateStreams(String)
     */
    public void updateStreams(String s) {
	if (s == null) {
	    streams.removeAll();
	    streams.setEnabled(false);
	    return;
	}
	//System.err.println("Updating streams: " + s);
	StringTokenizer strtok = 
	    new StringTokenizer(s, "|", false);
	int count = strtok.countTokens()-1;
	strtok.nextToken();
	for (int i = 0; i < count; i++) {
	    s = strtok.nextToken();
	    if (s.length() == 0)
		s = "Stream " + i;
	    StreamMenuItem smi = new StreamMenuItem(s, i);
	    smi.setSelected(true);
	    smi.addItemListener(this);
	    streams.add(smi);
	}
	if (count > 0)
	    streams.setEnabled(true);
	
	/*
	  for (int i = 0; i < v.size(); i++) {
	  String s = (String) v.elementAt(i);
	  if (s.length() == 0)
	  s = "Stream " + i;
	  StreamMenuItem smi = new StreamMenuItem(s, i);
	  smi.setSelected(false);
	  smi.addItemListener(this);
	  streams.add(smi);
	  }
	*/
    }
    
    /** 
     * A <code>JCheckBoxMenuItem</code> which also stores a stream
     * number.
     * @author Mike Jahr
     * @see OptionMenu
     */
    protected class StreamMenuItem extends JCheckBoxMenuItem {
	/** The associated stream number. */
	public int stream;
	
	/** 
	 * Creates a <code>StreamMenuItem</code> with the given
	 * properties.
	 */
	public StreamMenuItem(String name, int ins) {
	    super(name);
	    stream = ins;
	}
    }
    private static final String FONT_SUB        = "Fonts";
    private static final String SOURCE_FONT     = "Source sentence...";
    private static final String TARGET_FONT     = "Target sentence...";
    private static final String TABLE_FONT      = "Table...";
    private static final String COLOR_SELECTION = "Active word...";
    private static final String COLOR_LINKS     = "Active links...";
    private static final String COLOR_DEFAULT   = "Inactive words...";
    private static final String FOUR_UP         = "Display tables four-up";
    private static final String TWO_UP          = "Display tables two-up";
    private static final String ONE_UP          = "Display tables one-up";
    private static final String SHOW_ALL        = "Display both translations";
    private static final String SHOW_REFERENCE   = "Align reference translation";
    private static final String SHOW_MACHINE    = "Align machine translation";
    private static final String SHOW_VERT       = "Display alignment vertically";
    private static final String SHOW_HORIZ      = "Display alignment horizontally";
}// end class OptionMenu
