/*
 * Probability class
 * Noah A. Smith, 14 July 1999
 * Last edited 21 July 1999, NAS
 * Statistical Machine Translation Team at WS '99
 */

import java.lang.Double;
import java.lang.String;
import java.io.StreamTokenizer;
import java.io.IOException;

/** A probability with a "correctness." 
    @author Noah A. Smith
 */
public class Probability {
    private double prob;
    private boolean correct;
    
    /** Instantiate a probability with no given initial value. */
    public Probability(){
	prob = 0.0;
	correct = false;
    }

    /** Instantiate a probability with value 'p' and correctness 'c'. */
    public Probability(double p, boolean c){
	prob = p;
	correct = c;
    }
    
    /** Copy the probability 'P'. */
    public Probability(Probability P){
	prob = P.prob;
	correct = P.correct;
    }

    /** Return the probability value itself. */
    public double val(){
	return prob;
    }

    /** Return the correctness of the probable event. */
    public boolean correct(){
	return correct;
    }

    /** Assign argument's value to the Probability 'P'. */
    public void setEqualTo(Probability P){
	prob = P.prob;
	correct = P.correct;
    }

    /** Read in a probability UP TO AND INCLUDING AN END OF LINE from the StreamTokenizer 'st', given its correctness 'c'. */
    public String read(StreamTokenizer st, boolean c) throws FormatException{
	boolean loopDone = false;
	String temp = "";
	
	correct = c;
	try{
	    //	    while(!loopDone){
	    switch(st.nextToken()){
	    case st.TT_EOF:
		throw new FormatException("Unexpected end of file!  Pr  01");
	    case st.TT_WORD:
		temp = /*temp +*/ st.sval;
		break;
	    case st.TT_NUMBER:  throw new FormatException("Zut alors!  Pr  02");
	    case st.TT_EOL:  throw new FormatException("Unexpected end of line!  Pr  03");
	    }
	    
	    prob = Double.parseDouble(temp);

	    switch(st.nextToken()){
	    case st.TT_EOF:
		throw new FormatException("Unexpected end of file!  Pr  04");
	    case st.TT_WORD:  
		st.pushBack();
		return temp;
	    case st.TT_NUMBER:  throw new FormatException("Zut alors!  Pr  06");
	    }
	    //}
	    
	    return null;
	}
	catch(IOException ioe){
	    throw new FormatException(ioe.getMessage());
	}
	catch(NumberFormatException nfe){
	    return temp;
	}
    }

    /** Create a printable string representing the value and correctness. */
    public String toString(){
	return(new String(String.valueOf(prob) + " " + correct));
    }
}


