/* SPTableModel.java
 * =================
 * Mike Jahr, 7/16/99
 * Machine Translation group, WS99
 * Center for Language and Speech Processing
 * 
 * Last edited 7/21/99, Noah A. Smith
 * Table to display probabilities and strings.
 */

import javax.swing.table.*;
import java.util.Vector;
import java.util.Collections;
import java.awt.Color;

/** A model for a table of StringProbs, as seen in fertility, distortion, translation, and language model tables; keeps rows ordered.
    @author Noah A. Smith, Mike Jahr
    @see StringProb
    @see StringProbComparator
*/
public class SPTableModel extends AbstractTableModel {
    private Vector data; // vector of StringProbs
    private StringProbComparator SPC = new StringProbComparator();
    private String type;

    /** Create a new SPTableModel with label 't'. */
    public SPTableModel(String t) {
	super();
	data = new Vector();
	type = t;
    }
    
    /** Tell whether row numbered 'row' is correct. */
    public boolean rowIsCorrect(int row) {
	return ((StringProb) data.elementAt(row)).correct();
    }

    /* Sort the list by the default method. */
    /*    public void sort(){
	SPC.setCompMethod(StringProbComparator.PROBMETH, true);
	Collections.sort(data, SPC);
    }
    */
    /** Sort the list by the given method. 
     @see StringProbComparator
    */
    public void setSortMethod(int meth, boolean dir) {
	//*DEBUG*/ System.err.println("in SetSortMethod ... ");
	if(SPC == null){
	    //*DEBUG*/ System.err.println("SPC is null");
	    return;
	}
	switch(meth){
	case 0:
	    SPC.setCompMethod(StringProbComparator.STRINGMETH, dir);
	    break;
	case 1:
	    SPC.setCompMethod(StringProbComparator.PROBMETH, dir);
	    break;
	default:
	    SPC.setCompMethod(StringProbComparator.PROBMETH, dir);
	}
	//*DEBUG*/ System.err.println("Sorting ... ");
	Collections.sort(data, SPC);
    }

    /* Extensions of AbstractTableModel. */
    
    /** Return the number of rows. */
    public int getRowCount(){
	return data.size();
    }
    
    /** Return the number of columns. */
    public int getColumnCount(){
	return 2;
    }

    /** Return the color of the row numbered 'row'. */
    public Color rowColor(int row){
	return ((StringProb)data.elementAt(row)).getColor();
    }

    /** Return the actual table value at row 'row' and column 'column'. */
    public Object getValueAt(int row, int column){
	if(column == 0)
	    return ((StringProb)data.elementAt(row)).token();
	else if(column == 1)
	    return new Double(((StringProb)data.elementAt(row)).prob().val());
	else
	    return null;
    }
    
    /** Return the name of the column numbered 'column'. */
    public String getColumnName(int column){
	if(column == 0)
	    return type;
	else if(column == 1)
	    return "Probability";
	else
	    return "Unknown";
    }
    
    /** Add a StringProb 'sp' to the table. */
    public void add(StringProb sp){
	boolean ok = true;
	/*if(sp.token() == null) {
	    System.err.println("!!! I'm null");
	    }*/
	for(int i = 0; i < data.size(); i++){
	    if(((StringProb)data.elementAt(i)) == null) System.err.println("!!\t" + i);
	    if(((StringProb)data.elementAt(i)).token().equals(sp.token())){
		ok = false;
		//*DEBUG*/ System.err.println("Can't add StringProb, duplicate:  " + sp + " (already exists:)" + (StringProb)data.elementAt(i));
	    }
	}
	if(ok)
	    data.add(sp);
    }

    /** Return the number of rows in the table. */
    public int size(){
	return data.size();
    }
    
    /** Return the first correct StringProb element in the table. */
    public StringProb correct(){
	StringProb ret = new StringProb("", 0.0, false);
	for(int i = 0; i < data.size(); i++){
	    StringProb sprob = (StringProb)data.elementAt(i);
	    if(sprob.prob().correct())
		return sprob;
	}
	return ret;
    }
    
    /* Print the SPTM.
       public void print(){
       System.err.println(getColumnName(0) + "\t" + getColumnName(1));
       for(int i = 0; i < getRowCount(); i++){
       System.err.println(getValueAt(i, 0) + "\t" + getValueAt(i, 1));
       }
       }
    */

    /** Set the color of the StringProb with text 'text' to color 'color'. */
    public void setColor(String text, Color color){
	int i = 0;
	while(i < data.size() && (text.compareTo(((StringProb)data.elementAt(i)).toString()) != 0)){
	    //	    System.err.println((StringProb)data.elementAt(i) + " is not equal to " + text);
	    i++;
	}
	if(i < data.size()){
	    //	    System.err.println((StringProb)data.elementAt(i) + " having color set to " + color);
	    ((StringProb)data.elementAt(i)).setColor(color);
	}
    }

    /** Make all of the StringProbs in the table black. */
    public void makeAllBlack(){
	for(int i = 0; i < data.size(); i++)
	    ((StringProb)data.elementAt(i)).setColor(Color.black);
    }

    /** Set the color of the StringProb starting with text 'text' to color 'color'. */ 
    public void setColorStartsWith(String text, Color color){
	int i = 0;
	while(i < data.size() && !(((StringProb)data.elementAt(i)).toString().startsWith(text))){
	    i++;
	}
	if(i < data.size()){
	    //	    System.err.println((StringProb)data.elementAt(i) + " having color set to " + color);
	    ((StringProb) data.elementAt(i)).setColor(color);
	}
    }
	    
    /** Clear the SPTM. */
    public void clear(){
	data.clear();
    }
}
