/*
 * StringProb class
 * Noah A. Smith, 13 July 1999
 * Last edited 20 July 1999
 * Statistical Machine Translation Team at WS '99
 */

import java.lang.String;
import java.awt.Color;

/** A string with a probability attached.
    @author Noah A. Smith
    @see Probability
    @see String
*/
public class StringProb {
    private String word;
    private Probability prob;
    private Color color = Color.black;

    /** Create a StringProb word 'text' with probability 'tProb' and correctness 'c'. */
    public StringProb(String text, double tProb, boolean c){
	word = text;
	prob = new Probability(tProb, c);
    }

    /** Create a StringProb word 'text' with Probability (which includes correctness) 'p'. */
    public StringProb(String text, Probability p){
	word = text;
	prob = new Probability(p);
    }

    /** Return the "correctness" of a StringProb. */
    public boolean correct(){
	return prob.correct();
    }
    
    /** Return the StringProb token (word). */
    public String token(){
	return word;
    }
    
    /** Return the StringProb (t) probability. */
    public Probability prob(){
	return prob;
    }
    
    /** Return the StringProb token (word). */
    public String toString() {
	return word;
    }
    
    /** Set the color of this StringProb for when it is displayed in a table. */
    public void setColor(Color c){
	color = c;
    }
    
    /** Return the color of the StringProb. */
    public Color getColor(){
        return color;
    }
}
