/*
 * StringProbComparator class
 * Noah A. Smith, 16 July 1999
 * Last edited 16 July 1999
 * Statistical Machine Translation Team at WS '99
 */

import java.util.*;

/** Used by SPTableModel to sort StringProbs.
    @author Noah A. Smith
    @see SPTableModel
    @see StringProb
*/
public class StringProbComparator implements Comparator{
    public static final int STRINGMETH = 0; /** For sorting alphabetically by the String. */
    public static final int PROBMETH = 1; /** For sorting numerically by the Probability. */
    private int compMethod = PROBMETH;
    private boolean compDir = true;
    
    /** Instantiate a new StringProbComparator. */
    public StringProbComparator(){
    }
    
    /** Perform comparison of 'o1' and 'o2' based on current method of comparison. */
    public int compare(Object o1, Object o2){
	StringProb sp1 = (StringProb)o1;
	StringProb sp2 = (StringProb)o2;
	int ret = 0;
	//*DEBUG*/ System.err.println("compare(" + sp1 + " <=> " + sp2 + ")");
	switch(compMethod){
	case(STRINGMETH):
	    if(compDir)
		ret = sp1.token().compareTo(sp2.token());
	    else
		ret = -(sp1.token().compareTo(sp2.token()));
	    break;
	case(PROBMETH):
	    if(sp1.prob().val() < sp2.prob().val())
		ret = 1;
	    else if(sp1.prob().val() > sp2.prob().val())
		ret = -1;
	    else // they must be equal
		ret = 0;
	    break;
	}
	if(!compDir)
	    ret = -ret;
	//*DEBUG*/ System.err.println("\t" + ret);
	return ret;
    }

    /** Determine if the object is a StringProbComparator. */
    public boolean equals(Object o){
	return(o.getClass().getName().equals("StringProbComparator"));
    }
    
    /** Compare by method 'm' with direction 'dir' (true is default - A-Z or 1-0, false reverses the order). */
    public void setCompMethod(int m, boolean dir){
	if(m == STRINGMETH || m == PROBMETH){
	    compMethod = m;
	    compDir = dir;
	}
	else
	    System.err.println("StringProbComparator:  Cannot set comparison method to " + m + "!");
    }
}
    
