/*
 * TargetWord class
 * Noah A. Smith, 13 July 1999
 * Last edited 21 July 1999, NAS
 * Statistical Machine Translation Team at WS '99
 */

import java.util.Hashtable;
import java.io.StreamTokenizer;
import java.io.IOException;
import java.lang.Integer;
import java.util.Vector;

/** A Word in the target language (the one you translated to). 
    @author Noah A. Smith
    @see Word
*/
public class TargetWord extends Word { 
    private SPTableModel fertilities = new SPTableModel("Fertility");
    private SPTableModel distortions = new SPTableModel("Distortion");

    /** Instantiate target word 'text' in position 'i' of the target sentence with Web 'w'. */
    public TargetWord(String text, int i, Web w){
	super(text, i, w);
	//setMnemonic(java.awt.event.KeyEvent.VK_F1 + i);
    }
    
    /** True. */
    public boolean isTarget() {
	return true;
    }
    
    /** Add fertility 'fert' with Probability 'prob' to the list of fertilities. */
    private void addFertility(int fert, Probability prob){
	fertilities.add(new StringProb(String.valueOf(fert), prob));
    }

    /** Add distortion 'dist' with Probability 'prob' to the list of fertilities. */
    private void addDistortion(String dist, Probability prob){
	distortions.add(new StringProb(dist, prob));
    }
    
    /** Return the "correct" fertility or -1 if there isn't one. 
	private int correctFertility(){
	return Integer.decode(fertilities.correct().token()).intValue();
	}
    */

    /* Return the "correct" distortion or -1 if there isn't one. 
       private int correctDistortion(){
       return Integer.decode(fertilities.correct().token()).intValue();
       }
    */

    /** Read in a fertility from StreamTokenizer 'st' with correctness 'corr'. */
    /*       Format is:
	     <target word position> { <fertility> <d(fertility | target word position)> <newline> }
	     where {} indicate what is read. */
    public void readFertility(StreamTokenizer st, boolean corr) throws FormatException{
	int fert = -1;
	Probability prob = new Probability();
	try{
	    switch(st.nextToken()){
	    case st.TT_EOF:  throw new FormatException("Unexpected end of file!  TW  01");
	    case st.TT_NUMBER:  throw new FormatException("Zut alors!  TW  02");
	    case st.TT_EOL:  throw new FormatException("Unexpected end of line!  TW  03");
	    case st.TT_WORD:
		fert = Integer.parseInt(st.sval);
	    }
	    
	    prob.read(st, corr);
	}
	catch(IOException ioe){
	    throw new FormatException(ioe.getMessage());
	}
	//*DEBUG*/ System.err.println("Read fertility:  " + fert + " " + prob + " " + corr);
	addFertility(fert, prob);
    }

    /** Read in a distortion from StreamTokenizer 'st' with  correctness 'corr'. */ 
    /*Format is:
      <target word position> { <source word position> <d(source word position | target word position)> <newline> }
      where {} indicate what is read. */
    public void readDistortion(StreamTokenizer st, boolean corr) throws FormatException{
	int dist = -1, tlen = -1, slen = -1;
	Probability prob = new Probability();

	try{
	    switch(st.nextToken()){
	    case st.TT_EOF:  throw new FormatException("Unexpected end of file!  TW  04");
	    case st.TT_NUMBER:  throw new FormatException("Zut alors!  TW  05");
	    case st.TT_EOL:  throw new FormatException("Unexpected end of line!  TW  06");
	    case st.TT_WORD:
		dist = Integer.parseInt(st.sval);
	    }
	    switch(st.nextToken()){
	    case st.TT_EOF:  throw new FormatException("Unexpected end of file!  TW  07");
	    case st.TT_NUMBER:  throw new FormatException("Zut alors!  TW  08");
	    case st.TT_EOL:  throw new FormatException("Unexpected end of line!  TW  09");
	    case st.TT_WORD:
		tlen = Integer.parseInt(st.sval);
	    }
	    switch(st.nextToken()){
	    case st.TT_EOF:  throw new FormatException("Unexpected end of file!  TW  10");
	    case st.TT_NUMBER:  throw new FormatException("Zut alors!  TW  11");
	    case st.TT_EOL:  throw new FormatException("Unexpected end of line!  TW  12");
	    case st.TT_WORD:
		slen = Integer.parseInt(st.sval);
	    }
	

	    prob.read(st, corr);
	}
	catch(IOException ioe){
	    throw new FormatException(ioe.getMessage());
	}
	//*DEBUG*/ System.err.println("Read distortion:  " + dist + " " + prob + " " + corr);
	/*addDistortion("pos " + index + 
		      " -> pos " + dist + 
		      " | lengths " + slen + 
		      " (source) and " + tlen 
		      + " (target)", prob); */
	addDistortion("" + index + " -> " + dist +
		      " | s" + slen + ", t" + tlen, prob);
    }

    /** Return this Word's fertility table information. */
    public SPTableModel getFertilities(){
	return fertilities;
    }
    
    /** Return this Word's distortion table information. */
    public SPTableModel getDistortions(){
	return distortions;
    }

    /** Update the colors in all the tables for this Word. */
    public void updateColors(){
	updateTranslationColors();
	updateFertilityColors();
	updateDistortionColors();
    }

    /** Update the colors of the items in the distortion table so that they reflect the alignments to which they belong. */
    protected void updateDistortionColors(){
	String dText;
	distortions.makeAllBlack();
	//	System.err.println("UPDATE");
	for(int i = 0; i < links.size(); i++){
	    dText = "" + index + " -> " + String.valueOf(((Link)links.elementAt(i)).source().index());
	    distortions.setColorStartsWith(dText, ((Link)links.elementAt(i)).activeColor);
	}
    }

    /** Update the colors of the items in the fertility table so that they reflect the alignments to which they belong. */
    protected void updateFertilityColors(){
	String fText;
	int ferts[] = new int[Web.MAX_ALIGNMENTS];
	boolean done[] = new boolean[Web.MAX_ALIGNMENTS];
	Vector colors;
	//	System.err.println("FERTILITY UPDATE");
	fertilities.makeAllBlack();
	for(int i = 0; i < Web.MAX_ALIGNMENTS; i++){
	    ferts[i] = -1;
	    done[i] = false;
	}
	for(int i = 0; i < links.size(); i++)
	    for(int j = 0; j < Web.MAX_ALIGNMENTS; j++)
		if(((Link)links.elementAt(i)).getTypes()[j])
		   if(((Link)links.elementAt(i)).source().isNull())
		       ferts[j] = 0;
		   else{
		       ferts[j]++;
		       if(ferts[j] == 0) ferts[j]++;
		   }
	for(int i = 0; i < Web.MAX_ALIGNMENTS; i++){
	    if(done[i] == false && ferts[i] >= 0){
		colors = new Vector();
		done[i] = true;
		colors.addElement(Web.getActiveColor(i));	    
		for(int j = i + 1; j < Web.MAX_ALIGNMENTS; j++)
		    if(!(done[j]) && ferts[j] == ferts[i]){
			colors.addElement(Web.getActiveColor(j));
			done[j] = true;
		    }
		fertilities.setColor(String.valueOf(ferts[i]), Web.mixColors(colors));
	    }
	}
    }
}


