/* Web.java
 * ========
 * Mike Jahr, 7/14/99
 * Machine Translation group, WS99
 * Center for Language and Speech Processing
 * 
 * Last edited 7/21/99, NAS
 * Visual representation of the links between source 
 * and target sentences.
 */

import javax.swing.*;
import javax.swing.border.BevelBorder;
import java.awt.*;
import java.awt.event.*;
import java.io.StreamTokenizer;
import java.io.IOException;
import java.util.Vector;
import java.util.Enumeration;
//import java.lang.Integer;

/**
 * Visual representation of the links between source and target sentences.
 * Stores alignment information
 * @author Mike Jahr
 * @see Sentence
 * @see Cairo
 */

public class Web extends JComponent implements MouseListener {
    
    ////////////////// STATIC MEMBERS //////////////////
    
    //public static final Color ACTIVE_WORD = new Color(255, 0, 50);
    //public static final Color ACTIVE_LINK = new Color(150, 0, 20);
    //public static final Color ACTIVE_CROSS = ACTIVE_LINK;
    public static final Color DEFAULT_COLOR = Color.darkGray;
    public static final Color HUMAN_WORD = new Color(15, 0, 150);
    public static final int DEFAULT_HEIGHT = 40;
    public static final int MAX_ALIGNMENTS = 3;
    private Color allColor = null;
    private static final Color activeColors[] = {
	new Color(255, 0, 0), 
	new Color(0, 0, 255), 
	new Color(0, 255, 0)
    };
    private static final Color passiveColors[] = {
	new Color(170, 120, 120), 
	new Color(120, 120, 170), 
	new Color(120, 170, 120)
    };
    public static Color getActiveColor(boolean a[]) {
	int i;
	Vector v = new Vector(MAX_ALIGNMENTS);
	
	for (i = 0; i < a.length; i++)
	    if (a[i]) v.add(activeColors[i]);
	return mixColors(v);
    }
    public static Color getPassiveColor(boolean a[]) {
	int i;
	Vector v = new Vector(MAX_ALIGNMENTS);
	
	for (i = 0; i < a.length; i++)
	    if (a[i]) v.add(passiveColors[i]);
	return mixColors(v);
    }
    public static Color getActiveColor(int c) {
	return activeColors[c];
    }
    public static Color getPassiveColor(int c) {
	return passiveColors[c];
    }
    public Color getAllColor() {
	return allColor; 
    }
    public static Color mixColors(Vector v) {
	int len = v.size();
	if (len == 1) return ((Color) v.elementAt(0));
	
	int r=0, g=0, b=0;
	Color c;
	Enumeration e = v.elements();
	
	while (e.hasMoreElements()) {
	    c = (Color) e.nextElement();
	    r += c.getRed();
	    b += c.getBlue();
	    g += c.getGreen();
	}

	return new Color(Math.round((float)r/len),
			 Math.round((float)g/len),
			 Math.round((float)b/len));
    }
    
    ////////////////// PRIVATE VARIABLES //////////////////
    
    //private Sentence source, target, human;
    private Translation trans;
    private Vector activeLinks, autoLinks, humanLinks;
    private static final double STEM_LENGTH = 0.12; // as fraction of
    // height
    private static final int MAX_STEM = 30;
    //private int bot, top;
    private Rectangle r;
    private Word activeWord = null;
    private ProbTable tt, dt, lt, ft;
    private Cairo master;
    private boolean showAltTrans = false, finished = false;
    private int axis;
    private Component margin = null;
    //private int upperTop, upperBot, lowerTop, lowerBot;
    
    ////////////////// CONSTRUCTOR //////////////////
    
    public Web(Translation inTrans, Cairo c, int inaxis) {
	//Sentence insou, Sentence intar, Sentence inhum, Cairo c, int inaxis) {
	super();
	
	/*source = insou;
	target = intar;
	human = inhum;*/
	trans = inTrans;
	master = c;
	axis = inaxis;
	
	setAlignmentX((float) 0.5);
	setAlignmentY((float) 0.5);
	setBorder(BorderFactory.createEmptyBorder());
	//setBorderPainted(false);
	//setFocusPainted(false);
	
	//setContentAreaFilled(false);
	//setMargin(new Insets(1,1,1,1));
	
	autoLinks = new Vector();
	humanLinks = new Vector();
	activeLinks = autoLinks;
	r = new Rectangle();
	addMouseListener(this);
    }
    
    ////////////////// DISPLAY METHODS //////////////////
    
    public void setMargin(Component inm) {
	margin = inm;
    }
    
    public void setAxis(int newAxis) {
	if (axis == newAxis)
	    return;
	axis = newAxis;
	setSize(getHeight(), getWidth());
    }
    public int getAxis() {
	return axis;
    }
    public Translation getTrans() {
	return trans;
    }
    public void swapTarget() {
	Container c = getParent();
	c.remove(margin);
	c.remove(trans.human);
	c.remove(trans.target);
	
	if (activeLinks == humanLinks) {
	    c.add(trans.target);
	    if (showAltTrans)
		c.add(trans.human);
	} else {
	    c.add(trans.human);
	    if (showAltTrans)
		c.add(trans.target);
	}

	c.add(margin);
	
	activateWord(null);
	activeLinks = (activeLinks == humanLinks)
	    ? autoLinks : humanLinks;
	
	//trans.target.invalidate();
	//trans.human.invalidate();
	master.validate();
	//c.repaint();
    }

    public Sentence getActiveSentence() {
	if (activeLinks == humanLinks)
	    return trans.human;
	
	return trans.target;
    }
    public Sentence getAltSentence() {
	if (activeLinks == autoLinks)
	    return trans.human;
	
	return trans.target;
    }
    public boolean isAltShown() {
	return showAltTrans;
    }
        
    public void setShowAlt(boolean inshow) {
	//*DBG*/System.err.println("setShowAlt(" + inshow + 
	//*DBG*/"); showAltTrans=" + showAltTrans);
	
	if (inshow == showAltTrans)
	    return;
	
	//master.updateTables(null);
	//activateWord(null);
	Container c = getParent();

	c.remove(margin);

	if (inshow) {
	    if (activeLinks == humanLinks)
		c.add(trans.target);
	    else
		c.add(trans.human);
	} else {
	    if (activeLinks == humanLinks)
		c.remove(trans.target);
	    else
		c.remove(trans.human);
	}
	
	c.add(margin);
	
	showAltTrans = inshow;
	c.invalidate();
	c.getParent().invalidate();
	//c.validate();
	master.validate();
    }
    
    /*
    public void setTargetTrans(int which) {
	if ((which == Sentence.TARGET) == (activeLinks == autoLinks))
	    return;

	super.remove(human);
	super.remove(target);
	
	if (which == Sentence.TARGET) {
	    activeLinks = autoLinks;
	    super.add(target);
	    super.add(human);
	} else {
	    activeLinks = humanLinks;
	    super.add(human);
	    super.add(target);
	}
    }
    */
    public void activateWord(Word newword) {
	if (newword != null)
	    if (newword == activeWord || newword.isHuman())
		return;
	
	Link l, newLink = null;
	Vector aLinks = new Vector();
	Color def = (newword != null && activeLinks == humanLinks && 
		     !newword.isTarget()) ? HUMAN_WORD : DEFAULT_COLOR;
	
	if (activeWord != null) {
	    Color bg = activeWord.getForeground();
	    if (activeWord.isSource())
		activeWord.setForeground(DEFAULT_COLOR);
	    else
		activeWord.setForeground(def);
	    activeWord.setBackground(bg);
	    //activeWord.setBorder(BorderFactory.createEmptyBorder());
	    activeWord.setBorderPainted(false);
	}
	if (newword != null) {
	    //newword.setForeground(ACTIVE_WORD);
	    //newword.setBorder(BorderFactory.createEtchedBorder(Color.white, ACTIVE_WORD));
	    newword.setForeground(newword.getBackground());
	    newword.setBackground(DEFAULT_COLOR);
	    newword.setBorderPainted(true);
	}
	//createBevelBorder(BevelBorder.RAISED, ACTIVE_WORD, ACTIVE_CROSS));
	Enumeration e = activeLinks.elements();
	//Word newTar = null;
	
	while (e.hasMoreElements()) {
	    l = (Link) e.nextElement();
	    if (activeWord == l.tword && newword != l.sword)
		l.sword.setForeground(DEFAULT_COLOR);
	    if (activeWord == l.sword && newword != l.tword)
		l.tword.setForeground(def);
	    if (newword == l.tword || newword == l.sword)
		aLinks.add(l);
	    /*l.sword.setForeground(l.activeColor);
	    if (newword == l.sword)
		newLink = l;
	    */
	}
	e = aLinks.elements();
	while (e.hasMoreElements()) {
	    l = (Link) e.nextElement();
	    if (newword == l.tword)
		l.sword.setForeground(l.activeColor);
	    else if (newword == l.sword)
		l.tword.setForeground(l.activeColor);
	}
	/*
	if (newLink != null)
	    newLink.tword.setForeground(newLink.activeColor);
	*/
	activeWord = newword;

	getParent().repaint();
	master.updateTables(newword);
    }
    
    ////////////////// SWING METHODS //////////////////
    
    public Dimension getPreferredSize() {
	if (axis == BoxLayout.X_AXIS)
	    return new Dimension(100, 20);
	else
	    return new Dimension(20, 100);
    }
    /*
    public Dimension getMaximumSize() {
	if (axis == BoxLayout.X_AXIS) 
	    return new Dimension(50000000, 400);
	else
	    return new Dimension(400, 50000000);
    }
    */
    public Dimension getMinimumSize() {
	if (axis == BoxLayout.X_AXIS) 
	    return new Dimension(50, 20);
	else
	    return new Dimension(20, 50);
    }
    public void paintComponent(Graphics g) {
	//super.paintComponent(g);
	if (activeLinks.size() == 0 || !finished) return;
	
	g.setColor(DEFAULT_COLOR);
	//Word aTar = null;
	boolean active = (activeWord == null);
	Link l, aLink = null;
	Container p;
	int tmp;
	Vector aLinks = new Vector();
	Enumeration e = activeLinks.elements();
	getBounds(r);
	
	if (axis == BoxLayout.X_AXIS) { // paint horizontal
	    final int lowerTop = (int) (((r.height * STEM_LENGTH) > MAX_STEM)
					? MAX_STEM : (r.height * STEM_LENGTH));
	    final int upperBot = r.height - lowerTop;
	    int sx, tx;
	    sx = tx = 0;
	    
    	    while (true) {
		
		if (!e.hasMoreElements()) {
		    if (active || aLinks.size() == 0) break;
		    e = aLinks.elements();
		    active = true;
		}
		l = (Link) e.nextElement();
		
		if (active)
		    g.setColor(l.activeColor);
		else {
		    if (activeWord == l.tword || activeWord == l.sword) {
			aLinks.add(l);
			continue;
			//aLink = l;
			//active = true;
			//aTar = l.tword;
			//g.setColor(l.activeColor);
		    } else {
			g.setColor(l.passiveColor);
		    }
		}
		
		p = l.sword.getParent();
		if (p == null) {
		    if (l.sword.index != -1)
			continue;
		} else
		    sx = p.getX() - r.x + p.getWidth() / 2;
		
		p = l.tword.getParent();
		if (p == null) {
		    if (l.tword.index != -1)
			continue;
		} else
		    tx = p.getX() - r.x + p.getWidth() / 2;
		
		if (l.tword.index == -1 || l.sword.index == -1) {
		    if (l.sword.index > -1) {
			g.drawLine(sx, 0, sx, lowerTop);
			g.drawLine(sx+1, 0, sx+1, lowerTop);
		    } else if (l.tword.index > -1) {
			g.drawLine(tx, upperBot, tx, r.height);
			g.drawLine(tx+1, upperBot, tx+1, r.height);
		    }
		} else {
		    g.drawLine(sx, 0, sx, lowerTop);
		    g.drawLine(sx+1, 0, sx+1, lowerTop);
		    g.drawLine(tx, upperBot, tx, r.height);
		    g.drawLine(tx+1, upperBot, tx+1, r.height);
		    g.drawLine(sx, lowerTop, tx, upperBot);
		    g.drawLine(sx+1, lowerTop, tx+1, upperBot);
		}
		/*
		if (active) {
		    active= false;
		    g.setColor(DEFAULT_COLOR);
		    } */
	    }
	    /*
	    if (aLink != null && aLink.tword.index != -1) {
		g.setColor(aLink.activeColor);
		tx = aLink.tword.getX() - r.x + aLink.tword.getWidth() / 2;
		g.drawLine(tx, upperBot, tx, r.height);
		g.drawLine(tx+1, upperBot, tx+1, r.height);
	    }
	    */
	} else { // paint vertical
	    final int lowerTop = (int) (((r.width * STEM_LENGTH) > MAX_STEM)
					? MAX_STEM : (r.width * STEM_LENGTH));
	    final int upperBot = r.width - lowerTop;
	    int sy, ty;
	    sy = ty = 0;
	    
    	    while (true) {
		if (!e.hasMoreElements()) {
		    if (active || aLinks.size() == 0) break;
		    e = aLinks.elements();
		    active = true;
		}
		l = (Link) e.nextElement();
		
		if (active)
		    g.setColor(l.activeColor);
		else {
		    if (activeWord == l.tword || activeWord == l.sword) {
			aLinks.add(l);
			continue;
			//aLink = l;
			//active = true;
			//aTar = l.tword;
			//g.setColor(l.activeColor);
		    } else {
			g.setColor(l.passiveColor);
		    }
		}
		
		p = l.sword.getParent();
		if (p == null) {
		    if (l.sword.index != -1)
			continue;
		} else
		    sy = p.getY() - r.y + p.getHeight() / 2;
		
		p = l.tword.getParent();
		if (p == null) {
		    if (l.tword.index != -1)
			continue;
		} else
		    ty = p.getY() - r.y + p.getHeight() / 2;
		
		if (l.tword.index == -1 || l.sword.index == -1) {
		    if (l.sword.index > -1) {
			g.drawLine(0, sy, lowerTop, sy);
			g.drawLine(0, sy+1, lowerTop, sy+1);
		    } else if (l.tword.index > -1) {
			g.drawLine(upperBot, ty, r.width, ty);
			g.drawLine(upperBot, ty+1, r.width, ty+1);
		    }
		} else {
 		    g.drawLine(0, sy, lowerTop, sy);
 		    g.drawLine(0, sy+1, lowerTop, sy+1);
 		    g.drawLine(upperBot, ty, r.width, ty);
 		    g.drawLine(upperBot, ty+1, r.width, ty+1);
 		    g.drawLine(lowerTop, sy, upperBot, ty);
 		    g.drawLine(lowerTop, sy+1, upperBot, ty+1);
		}
	    }	    
	}
    }

/*
    public void recalc() {
	Rectangle r = getBounds();
	int lowerBot = bot = r.y + r.height - 1;
	int lowerTop = r.y + r.height * (1 - STEM_LENGTH);
	int upperTop = top = r.y + 1;
	int upperBot = r.y + r.height * STEM_LENGTH;

	Link l;
	Enumeration e = links.elements();
	while (e.hasMoreElements()) {
	    l = (Link) e.nextElement();
	    l.a.x = l.sword.getX() + l.sword.getWidth() / 2;
	    l.a.y = upperBot;
	    l.b.x = l.tword.getX() + l.tword.getWidth() / 2;
	    l.b.y = lowerTop;
	}
    }
*/

    public void clear() {
	finished = false;
	autoLinks.clear();
	humanLinks.clear();
	activateWord(null);
    }
    public void finish() {
	finished = true;
    }
    
    ////////////////// LISTENER METHODS //////////////////
    
    public void mousePressed(MouseEvent e) { }
    public void mouseReleased(MouseEvent e) { }
    public void mouseEntered(MouseEvent e) { }
    public void mouseExited(MouseEvent e) { }
    public void mouseClicked(MouseEvent e) {
	activateWord(null);
    }
    
    ////////////////// READ METHODS //////////////////
    
    public void readAlignment(StreamTokenizer st)
	throws FormatException {
	read(st, autoLinks);
	
	boolean used[] = new boolean[MAX_ALIGNMENTS];
	Vector v = new Vector();
	boolean lb[];
	Link l;
	Enumeration e = autoLinks.elements();
	while (e.hasMoreElements()) {
	    l = (Link) e.nextElement();
	    lb = l.getTypes();
	    for (int i = 0; i < MAX_ALIGNMENTS; i++) {
		if (lb[i] && !used[i]) {
		    used[i] = true;
		    v.add(activeColors[i]);
		}
	    }
	}
	allColor = mixColors(v);
    }
    
    public void readHumanAlignment(StreamTokenizer st) 
	throws FormatException {
	read(st, humanLinks);
	//*DBG*/System.err.println(trans.human.toString());
    }
    
    ////////////////// PRIVATE METHODS //////////////////
    
    private void read(StreamTokenizer st, Vector v) throws FormatException {
	boolean done = false;
	Word sw = null;
	Word tw = null;
	int pos = 0, n = -1;
	Link l = null;
	int tempint = 0;

	v.clear();
	
	st.eolIsSignificant(true);
	
	try {
	    while (!done) {
		switch (st.nextToken()) {
		case st.TT_EOF:
		    throw new FormatException("Unexpected end of file.");
		case st.TT_WORD:
		    if (st.sval.startsWith("</")) {
			done = true;
			break;
		    }
		    else{
			tempint = Integer.parseInt(st.sval);
			//DBG*/System.err.println("Web got token: " + tempint);
			switch (pos++) {
			case 0:
			    if (v == humanLinks)
				tw = trans.human.getWordAt(tempint);
			    else
				tw = trans.target.getWordAt(tempint);
			    break;
			case 1:
			    sw = trans.source.getWordAt(tempint);
			    break;
			case 2:
			    n = tempint;
			    break;
			default:
			    throw new FormatException("Unexpected token "
						      + "in alignment: " + st.sval);
			}
			break;
		    }
		case st.TT_NUMBER:  throw new FormatException("Zut alors!  Wb  01");
		case st.TT_EOL:
		    if (pos == 2)
			addLink(v, sw, tw, 0);
		    //v.add(new Link(sw, tw, 0));
		    else if (pos == 3)
			addLink(v, sw, tw, n);
		    //v.add(new Link(sw, tw, n));
		    //DBG*/System.err.println("Web adding link: " + sw.getText() +
		    //			      " to " + tw.getText());
		    tw = null;
		    sw = null;
		    pos = 0;
		    break;
		}
	    }
	} catch (IOException ioe) {
	    throw new FormatException(ioe.getMessage());
	} finally {
	    st.eolIsSignificant(true);
	    Enumeration e = v.elements();
	    while (e.hasMoreElements()) {
		l = (Link) e.nextElement();
		l.updateColors();
		l.activeColor = getActiveColor(l.getTypes());
		l.passiveColor = getPassiveColor(l.getTypes());
	    }
	}
    }

    private void addLink(Vector v, Word sw, Word tw, int n) {
	Link l, found = null;
	Enumeration e = v.elements();
	while (e.hasMoreElements()) {
	    l = (Link) e.nextElement();
	    if (l.source() == sw && l.target() == tw) {
		found = l;
		break;
	    }
	}
	if (found == null)
	    v.add(found = new Link(sw, tw));
	
	found.addType(n);
    }
}
