/*
 * Word class
 * Noah A. Smith, 13 July 1999
 * Last edited 21 July 1999, NAS
 * Statistical Machine Translation Team at WS '99
 */

import java.awt.*;
import javax.swing.*;
import java.util.Vector;
import java.util.StringTokenizer;
import java.lang.String;
import java.io.StreamTokenizer;
import java.io.IOException;
import java.awt.event.*;
import java.lang.Integer;

/** A Word, displayed on the screen as a clickable text token. 
    @author Noah A. Smith
    @see JButton
*/
public class Word extends JButton {
    protected int index;
    protected SPTableModel translations = new SPTableModel("Translation"); 
    //protected String fullText[];
    //protected boolean currStreams[];
    //protected int numStreams;
    //protected int numShownStreams;
    //protected StringTokenizer stringTok;
    protected Vector links = new Vector();
    public static SPTableModel emptyFertTM = new SPTableModel("Fertility"); /** An empty fertility table. */
    public static SPTableModel emptyDistTM = new SPTableModel("Distortion"); /** An empty distortion table. */
    public static SPTableModel emptyTransTM = new SPTableModel("Translation"); /** An empty translation table */
    
    /** Creates a degenerate word. */
    public Word() {
	super();
	init();
    }
    /** Instantiate word 'text' in position 'i' of the sentence, with Web web. */
    /* the web must be notified when we are clicked so it can 
     * activate and deactivate accordingly. */
    public Word(String text, int i, final Web web) {
	super(text);//(text.replace('|', '\n')));
	//	System.err.println("---" + getText() + "---");
	/*
	stringTok = new StringTokenizer(text, "|", false);
	numStreams = stringTok.countTokens();
	//	System.err.println(numStreams);
	numShownStreams = numStreams;
	fullText = new String[numStreams];
	currStreams = new boolean[numStreams];	
	for(int j = 0; j < numStreams; j++){
	    fullText[j] = stringTok.nextToken();
	    currStreams[j] = true;
	}
	*/
	//	System.err.println("---" + getText() + "---");

	init();
	
	index = i;
	if (index == -1)
	    return;
	
	final Word word = this;

	addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		if (web != null)
		    web.activateWord(word);
	    }
	});
    }
    
    private void init() {
	setBorder(BorderFactory.createEmptyBorder());
	setBorder(BorderFactory.createEtchedBorder(Color.white, 
						   Web.DEFAULT_COLOR));
	setBorderPainted(false);
	setFocusPainted(false);
	setHorizontalTextPosition(CENTER);
	setAlignmentX((float) 0.5);
	setAlignmentY((float) 0.5);
	setForeground(Web.DEFAULT_COLOR);
    }

    /** Return the x coordinate of the Word. */
    public int getX() {
	Container c = getParent();
	if (c == null) return super.getX();
	return c.getX();
    }
    /** Return the y coordinate of the Word. */
    public int getY() {
	Container c = getParent();
	if (c == null) return super.getY();
	return c.getY();
    }
    
    /** Tell whether this is a SourceWord. */
    public boolean isSource() {
	return false;
    }
    /** Tell whether this is a TargetWord. */
    public boolean isTarget() {
	return false;
    }
    /** Tell whether this is a HumanWord (i.e., from a reference translation). */
    public boolean isHuman() {
	return false;
    }
    /** Tell whether this is the NullWord. */
    public boolean isNull() {
	return false;
    }
    
    /** Add a potential translation token 'text' along with its Probability 'tProb'. */
    private void addTranslation(String text, Probability tProb){
	translations.add(new StringProb(text, tProb));
    }

    /** Return the "correct" translation or an empty translation if there isn't one. 
	public StringProb correctTranslation(){
	return translations.correct();
	}
    */

    /** Return the location in the sentence of this word. */
    public int index(){
	return index;
    }

    /** Read in a translation, up to and including a newline character. */
    /*       Format is:
       <target word position> { (<source word position> | <source word>) <t(source word | target word)> <newline> }
       where {} indicate what is read. */
    public void readTranslation(StreamTokenizer st, Sentence translationSent, boolean corr) throws FormatException{
	String wrd = null, temp = "";
	Probability prob = new Probability();
	
	try{
	    //	    if(!corr){
	    switch(st.nextToken()){
	    case st.TT_NUMBER:  throw new FormatException("Zut alors!  Wd  00.1");
	    case st.TT_EOF:  throw new FormatException("Unexpected end of file!  Wd  01");
	    case st.TT_EOL:  throw new FormatException("Unexpected end of line!  Wd  02");
	    case st.TT_WORD:
		wrd = st.sval;
	    }
       
	     /*	    }
		    }
		    else{
		    switch(st.nextToken()){
		    
		    case st.TT_NUMBER:  throw new FormatException("Zut alors!  Wd  02.1");
		    case st.TT_EOF:  throw new FormatException("Unexpected end of file!  Wd  03");
		    case st.TT_EOL:  throw new FormatException("Unexpected end of line!  Wd  04");
		    case st.TT_WORD:  
		    if(Integer.parseInt(st.sval) < translationSent.length())
		    wrd = new String(translationSent.wordAt(Integer.parseInt(st.sval)).getText());
		    else
		    throw new FormatException("Translation maps to value outside sentence range!  Wd  02.01");
		    break;
		    }
		    
	     */
	    prob.read(st, corr);
	} 
	catch(IOException ioe){
	    throw new FormatException(ioe.getMessage());
	}
	//*DEBUG*/ System.err.println("Read translation:  " + wrd + " " + prob + " " + corr);
	addTranslation(wrd, prob);
    }

    /** Return this Word's translation table information. */
    public SPTableModel getTranslations(){
	return translations;
    }

    /** Return this Word's fertility table information. */
    public SPTableModel getFertilities(){
	return emptyFertTM;
    }
    
    /** Return this Word's distortion table information. */
    public SPTableModel getDistortions(){
	return emptyDistTM;
    }
    /*
    public void addStream(int i){
	if(i < 0)
	    return;
	if(i < numStreams)
	    currStreams[i] = true;
	numShownStreams++;
	refreshText();
    }
    
    public void removeStream(int i){
	if(i < 0)
	    return;
	if(i < numStreams && i >= 0)
	    currStreams[i] = false;
	numShownStreams--;
	refreshText();
    }
    
    protected void refreshText(){
	String temp = "";
	for(int j = 0; j < numStreams; j++)
	    if(currStreams[j]){
		if(!(temp.equals("")))
		    temp = temp + "\n";
		temp = temp + fullText[j];
	    }
	    else
		temp = temp + "NULL\n";
	setText(temp);
    }
    */
    /** Convert the Word to a printable String. */
    public String toString() {
	//return getText();
	//new Exception().printStackTrace();
	return "<" + getClass().getName() + ":\"" + getText() + "\":" +
	    index + ">";
    }

    /** Return the Word's text (the token) as a String. */
    public String getBareWord(){
	return getText();//fullText[0];
    }

    /** Add a Link to the Word's list of things it is linked to. */
    public void addLink(Link l){
	links.addElement(l);
    }

    /** Update the colors of the Word's translation tables. */
    public void updateColors(){
	updateTranslationColors();
    }

    /** Update the colors of the items in the translation table so that they reflect the alignments to which they belong. */
    protected void updateTranslationColors(){
	String tText = "";
	translations.makeAllBlack();
	//	System.err.println("TRANSLATION UPDATE");
	for(int i = 0; i < links.size(); i++){
	    if(isSource())
		tText = ((Link)links.elementAt(i)).target().getBareWord();
	    else if(isTarget())
		tText = ((Link)links.elementAt(i)).source().getBareWord();
	    translations.setColor(tText, ((Link)links.elementAt(i)).activeColor);
	}
    }
}



